;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rdmdef.asm
; Purpose      : ReadMem Application Default State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rdmdef'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Module Name : rdmDefaultStateManager
; Description : ReadMem Application Default State Manager.
; Assumptions : Display is cleared on first time entry into the state.
; Input(s)    : CORECurrentEvent  - system event to be processed
;               COREEventArgument - event extra information
; Output(s)   : None
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rdmDefaultStateManager

rdmDefaultStateManager:

                ; Set IYReg the address of the ReadMem ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RDM_STATEENTRY
                jr      Z, rdmDefaultStateStateEntryEvent

                ; Check if mode depress event.
                cp      A, #RDM_MODEDEPRESS
                jr      Z, rdmDefaultStateModeDepressEvent

                ; Check if crown set event.
                cp      A, #RDM_CROWNSET
                jr      Z, rdmDefaultPulledCrownEvent

                ; Check if crown CW event.
                cp      A, #RDM_CWPULSES
                jr      Z, rdmDefaultCWCrownEvent

                ; Check if crown CCW event.
                cp      A, #RDM_CCWPULSES
                jr      Z, rdmDefaultCCWCrownEvent
                
                ; Check if STOP/RESET event.
                cp      A, #RDM_STOPRESETDEPRESS
                jr      Z, rdmDefaultStopEvent
                
                ; Check if START/SPLIT event.
                cp      A, #RDM_STARTSPLITDEPRESS
                jr      Z, rdmDefaultStartEvent

                ret

                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

rdmDefaultPulledCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN SET
                ;
                ;**************************************************************

                ld      B, #RDMSETSTATE
                CORE_REQ_STATE_CHANGE

                ret

rdmDefaultStateStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************
                CORE_ENABLE_PULSE_MODE
                ld      L,[IY+RDMADDRLO]
                ld      H,[IY+RDMADDRHI]
                ld      [MemStart],HL

                
                ;-- Init the address --
                ld      A,[IY+RDMTYPE]
                cp      A,#0
                jr      NZ,LE1
                    ld  IX,#00000h
                    ld  B,#SEG_R
                    jr  LE5

LE1:
                cp      A,#1
                jr      NZ,LE2
                    ld  IX,#00000h
                    ld  B,#SEG_V
                    jr  LE5

LE2:
                cp      A,#2
                jr      NZ,LE3
                    ld  IX,#00000h
                    ld  B,#SEG_L
                    jr  LE5

LE3:
                cp      A,#3
                jr      NZ,LE4
                    ld  IX,#00000h
                    ld  B,#SEG_S
                    jr  LE5

LE4:
                cp      A,#4
                jr      NZ,LE5
                    ld  IX,#08000h
                    ld  B,#SEG_E
LE5:
                ld      [MemEnd],IX
                ld      [SegType],B
                
                car     Refresh                

                ret

rdmDefaultStateModeDepressEvent:
                ;**************************************************************
                ;
                ;                       MODE DEPRESS
                ;
                ;**************************************************************

                ld      HL,[MemStart]
                ld      [IY+RDMADDRLO],L
                ld      [IY+RDMADDRHI],H
                CORE_REQ_MODE_CHANGE_NEXT

                ret
                
rdmDefaultCWCrownEvent:
                ;**************************************************************
                ;
                ;                       Crown CW
                ;
                ;**************************************************************
                ld      HL,[MemStart]
                sub     HL,#4
                cp      HL,#0FFFEh
                jr      NZ,Lcw1
                    ld  HL,[MemEnd]
                    sub HL,#4
Lcw1:
                ld      [MemStart],HL
                car     Refresh
                
                ret

                
rdmDefaultCCWCrownEvent:
                ;**************************************************************
                ;
                ;                       Crown CCW
                ;
                ;**************************************************************
                ld      BA,[MemEnd]
                ld      HL,[MemStart]
                add     HL,#4
                cp      HL,BA
                jr      NZ,Lccw1
                    ld  HL,#00000h
Lccw1:
                ld      [MemStart],HL
                car     Refresh
                
                ret
                

rdmDefaultStopEvent:
                ;**************************************************************
                ;
                ;                       STOP/RESET
                ;
                ;**************************************************************
                ld      HL,[MemStart]
                inc     H
                ld      [MemStart],HL
                car     Refresh
                
                ret

rdmDefaultStartEvent:
                ;**************************************************************
                ;
                ;                       START/SPLIT
                ;
                ;**************************************************************
                ld      HL,[MemStart]
                dec     H
                ld      [MemStart],HL
                car     Refresh
                
                ret

                
;-------------------------------------------------------------
;-- Refresh the display --
Refresh:
                ld      IY, [CORECurrentASDAddress]
                ld      A,[IY+RDMTYPE]
                cp      A,#04   ;EEPROM?
                jr      NZ,Lr1
;-- Read EEPROM
                    ld  HL,[MemStart]
                    ld  [DBExternalMemoryAddress],HL
                    ld  HL,#Buffer
                    ld  [DBInternalMemoryAddress],HL
                    ld  HL,#04
                    ld  [DBLength],HL
                    car hwI2CEEPowerON
                    car dbReadExternalMemory
                    car dbCloseFile
                    jr  Lr2
;-- Read ROM, RAM, LCD, SFR                    
Lr1:            
                ld      B, #4
                ld      IY, [MemStart]
                ld      IX, #Buffer
                UTL_COPY_IYREG_TO_IXREG
Lr2:                
                car rdmDisplayBuffer
                ret
                
;-------------------------------------------------------------
;-- Display the buffer --                
rdmDisplayBuffer:
                ;-- Display the address in the segment area
                ld      BA,[MemStart]
                ld      HL,#SegDig34
                car     Hex2Car
                
                ld      BA,[MemStart]
                ld      A,B
                ld      HL,#SegDig12
                car     Hex2Car
                
                ld      IY, #SegMsg
                LCD_DISP_SEG_LINE_MSG

                ;-- Display the address of the line 1
                ld      BA,[MemStart]
                push    A
                
                ld      HL,#Car2Display
                car     Hex2Car
                
                ld      IY, #Car2Display
                ld      B, #2
                ld      IX, #LCDMAINDMLINE1COL1
                LCD_DISP_UNFORMATTED_SMALL_FIXED_WIDTH_DM_MSG

                ;-- Display the address of the line 2
                pop     A
                inc     A
                inc     A
                ld      HL,#Car2Display
                car     Hex2Car
                
                ld      IY, #Car2Display
                ld      B, #2
                ld      IX, #LCDMAINDMLINE2COL1
                LCD_DISP_UNFORMATTED_SMALL_FIXED_WIDTH_DM_MSG
                
                ;-- Display the containt of the buffer 
                ld      IY,#Buffer
                ld      L,#0
jmp0:                
                ld      A,[IY+L]
                cp      L,#00
                jr      NZ, jmp1
                    ld  IX, #LCDMAINDMLINE1COL17
                    jr  jmps
jmp1:                    
                cp      L,#01
                jr      NZ, jmp2
                    ld  IX, #LCDMAINDMLINE1COL30
                    jr  jmps
jmp2:
                cp      L,#02
                jr      NZ, jmp3
                    ld  IX, #LCDMAINDMLINE2COL17
                    jr  jmps
jmp3:
                cp      L,#03
                jr      NZ, jmps
                    ld  IX, #LCDMAINDMLINE2COL30
jmps:
                push    IY
                push    L
                
                ld      HL,#Car2Display
                car     Hex2Car
                
                ld IY, #Car2Display
                ld B, #2
                LCD_DISP_UNFORMATTED_SMALL_FIXED_WIDTH_DM_MSG
                
                pop     L
                pop     IY
                
                inc     L
                cp      L,#4
                jr      NZ,jmp0
                
                ret

;-- Hexadecimal to Caractere conversion --
;   A = Number to convert
;   HL= Address where the result will be stored (HI digit, LO digit)
;
Hex2Car:
                upck
                cp      A,#10
                jr      C,h2c1
                    inc A
h2c1:
                cp      B,#10
                jr      C,h2c2
                    inc B
h2c2:
                ld      [HL],B
                inc     HL
                ld      [HL],A
                ret

                
;-----------------------------                
Car2Display:    db DM5_0, DM5_0

MemStart:       dw  0000
MemEnd:         dw  0000

SegMsg:
SegType:        db  SEG_R, SEG_SPACE
SegDig12:       db  SEG_F, SEG_F
SegDig34:       db  SEG_4, SEG_0

;-- Memory Buffer --
Buffer:         db 00, 00, 00, 00, 00
        
