;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File Name    : rdmset.asm
; Purpose      : ReadMem Application Set State Manager
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;DO NOT MODIFY THE FOLLOWING SUBROUTINE DEFINITION;;;;;;;;;;;;;;;;;

                IF @DEF('SUBROUTINE')
                    UNDEF SUBROUTINE
                ENDIF
                DEFINE  SUBROUTINE      "'rdmset'"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                GLOBAL  rdmSetStateManager

rdmSetStateManager:

                ; Set IYReg the address of the ReadMem ASD.
                ld      IY, [CORECurrentASDAddress]

                ld      A, [CORECurrentEvent]

                ; Check if state entry event.
                cp      A, #RDM_STATEENTRY
                jr      Z, rdmSetStateEntryEvent

                ; Check if crown CW event.
                cp      A, #RDM_CWPULSES
                jr      Z, rdmSetCWCrownEvent

                ; Check if crown CCW event.
                cp      A, #RDM_CCWPULSES
                jr      Z, rdmSetCCWCrownEvent
                
                ; Check if crown Home event.
                cp      A, #RDM_CROWNHOME
                jr      Z, rdmSetPushedCrownEvent

                ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVENT HANDLERS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


rdmSetStateEntryEvent:
                ;**************************************************************
                ;
                ;                       STATE ENTRY
                ;
                ;**************************************************************
                CORE_ENABLE_PULSE_MODE
                
                car DisplayLine1
                car DisplayChoice
                
                ret
                
                
rdmSetCWCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN CW
                ;
                ;**************************************************************
                ld      A,[IY+RDMTYPE]
                inc     A
                cp      A,#5
                jr      NZ,LCw1
                    ld  A,#0
LCw1:           ld      [IY+RDMTYPE],A
                car     DisplayChoice
                ret
                
                
rdmSetCCWCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN CCW
                ;
                ;**************************************************************
                ld      A,[IY+RDMTYPE]
                dec     A
                cp      A,#255
                jr      NZ,LCCw1
                    ld  A,#4
LCCw1:          ld      [IY+RDMTYPE],A
                car     DisplayChoice
                ret
                
                
rdmSetPushedCrownEvent:
                ;**************************************************************
                ;
                ;                       CROWN HOME
                ;
                ;**************************************************************
                ld      B, #RDMDEFAULTSTATE
                CORE_REQ_STATE_CHANGE
                ret

                
;-- Display the line 1 --
;   Param = None
DisplayLine1:
                ld      IY, #MsgLine1
                LCD_DISP_FORMATTED_SMALL_FIXED_WIDTH_DM_MSG                
                ret
                
                
;-- Display the choice --
DisplayChoice:
                LCD_CLR_MAIN_DM_LINE2
                ld      IY, [CORECurrentASDAddress]
                ld      A,[IY+RDMTYPE]
                cp      A,#0
                jr      NZ,L1
                    ld  HL,#00000h
                    ld  IY,#MsgROM
                    jr  LDisp

L1:
                cp      A,#1
                jr      NZ,L2
                    ld  HL,#0F000h
                    ld  IY,#MsgRAM
                    jr  LDisp

L2:
                cp      A,#2
                jr      NZ,L3
                    ld  HL,#0F800h
                    ld  IY,#MsgLCD
                    jr  LDisp

L3:
                cp      A,#3
                jr      NZ,L4
                    ld  HL,#0FF00h
                    ld  IY,#MsgSFR
                    jr  LDisp

L4:
                cp      A,#4
                jr      NZ,LDisp
                    ld  HL,#00000h
                    ld  IY,#MsgEEPROM
LDisp:
                push    HL
                LCD_DISP_FORMATTED_SMALL_FIXED_WIDTH_DM_MSG
                
                pop     HL
                
                ld      IY, [CORECurrentASDAddress]
                ld      [IY+RDMADDRLO],L
                ld      [IY+RDMADDRHI],H

                ret

               
MsgLine1:
                dw      LCDMAINDMLINE1COL1
                db      4, DM5_M, DM5_E, DM5_M, DM5_QUESTION

MsgROM:
                dw      LCDMAINDMLINE2COL1
                db      4, DM5_GREATERTHAN, DM5_R, DM5_O, DM5_M

MsgRAM:
                dw      LCDMAINDMLINE2COL1
                db      4, DM5_GREATERTHAN, DM5_R, DM5_A, DM5_M
MsgLCD:
                dw      LCDMAINDMLINE2COL1
                db      4, DM5_GREATERTHAN, DM5_L, DM5_C, DM5_D
                
MsgSFR:
                dw      LCDMAINDMLINE2COL1
                db      4, DM5_GREATERTHAN, DM5_S, DM5_F, DM5_R
                
MsgEEPROM:
                dw      LCDMAINDMLINE2COL1
                db      7, DM5_GREATERTHAN, DM5_E, DM5_E, DM5_P, DM5_R, DM5_O, DM5_M

